SystemOrganization addCategory: #'Finder-Core'!
SystemOrganization addCategory: #'Finder-UI'!

Object subclass: #Finder
	instanceVariableNames: 'searchingString selectedMethod selectedClass methodsDictionary packagesSelection currentSearchMode environment resultDictionary useRegEx'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Finder-Core'!

!Finder commentStamp: 'BenjaminVanRyseghem 9/15/2010 11:17' prior: 0!
I'm the finderUI model in the MVC pattern.

I'm compute the search string and I construct a resultDictionary.
Then I throw events everywhere and my views are redrawn.!

!Finder class methodsFor: 'menu' stamp: 'AlainPlantec 2/14/2011 21:27'!
finderMenuOn: aBuilder 
	"I build a menu"
	<worldMenu>
	(aBuilder item: #Finder)
		action: [self open];
		parent: #Tools;
		help: 'Looking for somethoing ?';
		icon: self icon! !

!Finder class methodsFor: 'menu' stamp: 'AlainPlantec 2/14/2011 21:26'!
icon
	"My menu icon"
	^ ThemeIcons smallInfoIcon ! !

!Finder class methodsFor: 'menu' stamp: 'AlainPlantec 2/14/2011 21:25'!
open
	^ self new open! !

!Finder methodsFor: 'private' stamp: 'BenjaminVanRyseghem 3/4/2011 12:06'!
computeListOfClasses: aString
"Compute in the case I'm searching class names"
	| list result originalList |
	originalList := self packagesSelection.
	result := OrderedCollection new.
	
	'Scanning'
		displayProgressAt: Display center
			from: 0 to: originalList size
			during: [:bar |  	
				self useRegEx
					ifTrue: [ originalList doWithIndex: [:each :i | 
							bar value: i.
							(each name substrings anySatisfy: [:sub | sub matchesRegexIgnoringCase: aString])
								ifTrue: [
									result add: each.
									result add: each classSide]]]
					ifFalse: [ 
						originalList doWithIndex: [:each :i | 
							bar value: i.
							(each name includesSubstring: aString caseSensitive: false)
								ifTrue: [
									result add: each.
									result add: each classSide]]]].
	^result! !

!Finder methodsFor: 'private' stamp: 'StephaneDucasse 1/28/2011 16:38'!
computeWithMethodFinder: aString
	"Compute the selectors for the single example of receiver and args, in the very top pane" 

	| data result resultArray dataStrings mf dataObjects aa statements |
	((aString includes: $.) not) ifTrue: [^#()].
	data := aString.
	"delete t
 railing period. This should be fixed in the Parser!!"
 	[data last isSeparator] whileTrue: [data := data allButLast]. 
	data last = $. ifTrue: [data := data allButLast]. 	"Eval"
	mf := MethodFinder new.
	data := mf cleanInputs: data.	"remove common mistakes"
	[dataObjects := Compiler evaluate: '{', data, '}'] on:
SyntaxErrorNotification do: [:e | 
		self inform: 'Syntax Error: ', e errorMessage. 
		self contents: (e errorCode allButFirst allButLast).
		^#().]. "#( data1 data2 result )"
	statements := (Compiler new parse: 'zort ' , data in: Object notifying: nil)
						body statements select: [:each | (each isKindOf: ReturnNode) not].
 	dataStrings := statements collect:
				[:node | String streamContents:
					[:strm | (node isMessage) ifTrue: [strm nextPut: $(].
					node shortPrintOn: strm.
					(node isMessage) ifTrue: [strm nextPut: $)].]].
	dataObjects size < 2 ifTrue: [self inform: 'If you are giving an example of receiver, \args, and result, please put periods between the parts.\Otherwise just type one selector fragment' withCRs. ^#()].
 	dataObjects := Array with: dataObjects allButLast with: dataObjects last. "#( (data1
  data2) result )" 
	result := mf load: dataObjects; findMessage.
	(result first beginsWith: 'no single method') ifFalse: [
		aa := self testObjects: dataObjects strings: dataStrings.
		dataObjects := aa second.  dataStrings := aa third].
	resultArray := self listFromResult: result. 
	resultArray isEmpty ifTrue: [self inform: result first].

	dataStrings size = (dataObjects first size + 1) ifTrue:
		[resultArray := resultArray collect: [:expression | | newExp |
		newExp := expression.
		dataObjects first withIndexDo: [:lit :i |
			newExp := newExp copyReplaceAll: 'data', i printString
							with: (dataStrings at: i)].
		newExp, ' --> ', dataStrings last]].

 	^ resultArray! !

!Finder methodsFor: 'private' stamp: 'BenjaminVanRyseghem 2/17/2011 11:44'!
constructClassNamesDictionary
"Construct the dictionary in the case I'm searching in class names"
	| result listOfClasses listOfAssociations |

	listOfClasses := self computeListOfClasses: self searchingString.
	result := Dictionary new.
	listOfClasses do: [:each | result at: each put: (each methodDict keys sort: [:a :b | a < b])].
	self resultDictionary: result.! !

!Finder methodsFor: 'private' stamp: 'BenjaminVanRyseghem 2/23/2011 15:28'!
constructDictionary
"I construct the good dictionary regarding the search mode"

	| symbol |
	self searchingString isEmpty ifTrue: [ ^ self resultDictionary: Dictionary new].
	symbol := self currentSearchMode.
	symbol = #Selectors
		ifTrue: [ ^self constructDictionaryWithMessagesNameSearch: self searchingString].
	symbol = #Classes
		ifTrue: [ ^self constructClassNamesDictionary].
	symbol = #Source
		ifTrue: [ ^self constructSourceDictionary].
	symbol = #Examples
		ifTrue: [ ^self constructDictionaryWithMethodFinder: self searchingString].
	self error: 'Error with radio buttons'! !

!Finder methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/15/2010 23:02'!
constructDictionaryWithMessagesNameSearch: aString
"Construct dictionary when searching for selector"
	| result listOfMethods |
	
	result := Dictionary new.
	listOfMethods := self messagesNameSearch: aString.
	listOfMethods do: [:method || key value |
			key := method selector.
			value := method methodClass.
			(result includesKey: (key))
				ifTrue: [ (result at: (key)) add: value]
				ifFalse: [ result at: key put: (OrderedCollection new add: value; yourself)]].
	self resultDictionary: result.! !

!Finder methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/15/2010 23:02'!
constructDictionaryWithMethodFinder: aString
"construct dictionary when searching patterns"
	| result listOfStrings listOfSelectors |
	
	result := Dictionary new.
	listOfStrings := self computeWithMethodFinder: aString.
	listOfSelectors := listOfStrings collect: [:each | self findSelector: each].
	self packagesSelection do: [ :class | 
		class methodDict do: [: method || index |
			(( index := listOfSelectors indexOf: (method selector)) = 0) not
				ifTrue: [ | key value receiver receiverString|
					key := listOfStrings at: index.
					value := method methodClass.
					(result includesKey: (key))
						ifTrue: [ (result at: (key)) add: value]
						ifFalse: [ result at: key put: (OrderedCollection new add: value; yourself)]]]].
	self resultDictionary: result.! !

!Finder methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/15/2010 23:03'!
constructSourceDictionary
"construct dictionary when searching source"
	| result listOfMethods |
	
	result := Dictionary new.
	listOfMethods := self sourceSearch: self searchingString.
	listOfMethods do: [:method || key value |
			key := method selector.
			value := method methodClass.
			(result includesKey: (key))
				ifTrue: [ (result at: (key)) add: value]
				ifFalse: [ result at: key put: (OrderedCollection new add: value; yourself)]].
	self resultDictionary: result.! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/11/2010 23:54'!
currentSearchMode
"Getter"
"I shoud answer a Symbol in :
	- #Selectors
	- #Classes
	- #Source
	- #Examples"
	
	 ^ currentSearchMode! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/11/2010 23:54'!
currentSearchMode: aSymbol
"Setter"
"If #Examples is selected, I disable the RegEx checkbox"
"Then I rebuild the resultDictionary"

	currentSearchMode := aSymbol.
		aSymbol == #Examples
			ifTrue: [ self disableUseRegEx]
			ifFalse:[ self enableUseRegEx].
	self constructDictionary! !

!Finder methodsFor: 'initialize-release' stamp: 'BenjaminVanRyseghem 11/15/2010 13:40'!
defaultEnvironment
	 ^ Smalltalk allClassesAndTraits.! !

!Finder methodsFor: 'initialize-release' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
defaultPackagesSelection
	 ^ self environment! !

!Finder methodsFor: 'initialize-release' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
defaultString
	^''! !

!Finder methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/15/2010 23:03'!
disableUseRegEx
"send a disable useRegEx dropbox event"
	^ self triggerEvent: #disableUseRegEx! !

!Finder methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/15/2010 23:04'!
enableUseRegEx
"send a enable useRegEx dropbox event"
	^ self triggerEvent: #enableUseRegEx! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/18/2010 14:46'!
environment
"Getter"
"I should be a classes collection"
	 ^ environment reject: [:each |  each isObsolete].! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/15/2010 11:50'!
environment: aCollection
"Setter"
	 environment := aCollection! !

!Finder methodsFor: 'private' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
findSelector: aString
	"Answer the selector of aString."

	| example tokens |
	example := aString.
	tokens := Scanner new scanTokens: example.
	tokens size = 1 ifTrue: [^ tokens first].
	tokens first == #'^' ifTrue: [^ nil].
	(tokens second includes: $:) ifTrue: [^ example findSelector].
	Symbol hasInterned: tokens second ifTrue: [:aSymbol | ^ aSymbol].
	^ nil! !

!Finder methodsFor: 'initialize-release' stamp: 'BenjaminVanRyseghem 9/14/2010 18:52'!
initialize
	super initialize.
	searchingString := self defaultString.
	environment := self defaultEnvironment.
	packagesSelection := self defaultPackagesSelection.
	resultDictionary := Dictionary new.
	currentSearchMode := #Selectors.
	useRegEx := false.
	"SystemChangeNotifier uniqueInstance
		notify: self 
		ofSystemChangesOfItem: #category
		using: #resetEnvironment"! !

!Finder methodsFor: 'checkbox' stamp: 'BenjaminVanRyseghem 11/11/2010 23:31'!
isPatternSymbol
"answer if the current mode is Selectors"
	^self currentSearchMode = #Examples! !

!Finder methodsFor: 'checkbox' stamp: 'BenjaminVanRyseghem 9/15/2010 23:23'!
isSelectorsSymbol
"answer if the current mode is Selectors"
	^self currentSearchMode = #Selectors! !

!Finder methodsFor: 'checkbox' stamp: 'BenjaminVanRyseghem 9/15/2010 23:23'!
isSourceSymbol
"answer if the current mode is Selectors"
	^self currentSearchMode = #Source! !

!Finder methodsFor: 'private' stamp: 'BenjaminVanRyseghem 11/11/2010 23:34'!
listFromResult: resultOC
	"ResultOC is of the form #('(data1 op data2)' '(...)'). Answer a sorted array."

	(resultOC first beginsWith: 'no single method') ifTrue: [^ #()].
	^ resultOC sort: [:a :b | 
		(a copyFrom: 6 to: a size) < (b copyFrom: 6 to: b size)].
! !

!Finder methodsFor: 'private' stamp: 'BenjaminVanRyseghem 3/4/2011 12:05'!
messagesNameSearch: aString
"I'm search for selectors"
	| list result originalList bool starting |
	originalList := self packagesSelection.
	result := OrderedCollection new.

	'Scanning'
		displayProgressAt: Display center
		from: 0 to: originalList size
		during:[ :bar | 
			self useRegEx
				ifTrue: [
					originalList doWithIndex: [:class :i || temp |
						bar value: i.
						temp := class methodDict values asOrderedCollection select: [:method | (method selector asString) substrings anySatisfy: [:sub | sub matchesRegexIgnoringCase: aString]].
						temp addAll: (class classSide methodDict values asOrderedCollection select: [:method | (method selector asString) substrings anySatisfy: [:sub | sub matchesRegexIgnoringCase: aString]]).
						result addAll: temp]]
				ifFalse: [
					originalList doWithIndex: [:class :i || temp |
						bar value: i.
						temp := class methodDict values asOrderedCollection select: [:method | method selector asString includesSubstring: aString caseSensitive: false].
						temp addAll: (class classSide methodDict values asOrderedCollection select: [:method | method selector asString includesSubstring: aString caseSensitive: false]).
						result addAll: temp]]].
	^result! !

!Finder methodsFor: 'display' stamp: 'AlainPlantec 2/14/2011 21:24'!
open
	(self uiClass on: self) open! !

!Finder methodsFor: 'display' stamp: 'AlainPlantec 2/13/2011 21:36'!
openInWorld

	(self uiClass on: self) open! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/15/2010 11:51'!
packagesSelection
"Getter"
"I should be a selection of classes"
	^packagesSelection! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/17/2010 02:51'!
packagesSelection: aCollection
"Setter"
"I cha.nge the classes selection, then I rebuild the result dictionary"
	packagesSelection :=  aCollection.
	self constructDictionary.! !

!Finder methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/15/2010 23:05'!
resetEnvironment
"I send aan event when I reset the environment"
	self environment: self defaultEnvironment.
	self triggerEvent: #resetEnvironment.! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/15/2010 11:52'!
resultDictionary
"Getter"
"I should answer a dictionary"
	^resultDictionary! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 2/23/2011 15:28'!
resultDictionary: aDictionary
"Setter"
"When the result dictionary is update, I send an event through the whole universe"

	resultDictionary := aDictionary.
	self triggerEvent: #updateResultDictionary.
	self selectedMethod: nil.
	self selectedClass: nil.
	self triggerEvent: #updateSourceCode.! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/15/2010 11:20'!
searchingString
"Getter"
	^searchingString! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 2/23/2011 15:28'!
searchingString: aString
"When the search string is modified (in fact, when I make a search) I rebuild the result dictionary"

	searchingString := aString.
	self constructDictionary.! !

!Finder methodsFor: 'accessing' stamp: 'bvr 9/19/2010 18:27'!
searchingStringSilently: aString
	searchingString := aString.
! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/15/2010 11:20'!
selectedClass
"Getter"
	^selectedClass! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/15/2010 11:55'!
selectedClass: aClass
"When the class seletion is changed, I send an event to UI"
	selectedClass := aClass.
	self triggerEvent: #updateSelectedClass
	! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/15/2010 11:55'!
selectedMethod
"Getter"
"I should answer a CompiledMethod"
	^selectedMethod! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/15/2010 11:59'!
selectedMethod: aMethod
"When the method seletion is changed, I send an event to UI"
	selectedMethod := aMethod.
	self triggerEvent: #updateSelectedMethod! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/11/2010 23:55'!
selection: anObject
"anObject is a selection holder"
"Depending of the value of currentSearchMode, I fill selectedMethod and SelectedClass with the good items"
"Then, I update the source code text area"

	| path method class |
	(anObject isNil or: [anObject selectedNodePath isNil]) ifTrue: [
		self selectedClass: nil.
		self selectedMethod: nil.
		^self].
	path := anObject selectedNodePath.
	self currentSearchMode = #Selectors
		ifTrue: [
			method := path first.
			class := path at: 2 ifAbsent:[nil]].
	self currentSearchMode = #Classes
		ifTrue: [
			class := path first.
			method := path at: 2 ifAbsent:[nil]].
	self currentSearchMode = #Source
		ifTrue: [
			method := path first.
			class := path at: 2 ifAbsent:[nil]].
	self currentSearchMode = #Examples
		ifTrue: [
			method := path first.
			class := path at: 2 ifAbsent:[nil]].
	self selectedClass: (class
							ifNil: [nil] 
							ifNotNilDo: [:each | each item]).
	self selectedMethod: (method
							ifNil: [nil]
							ifNotNilDo: [:each | each item]).
	self triggerEvent: #updateSourceCode! !

!Finder methodsFor: 'private' stamp: 'BenjaminVanRyseghem 11/15/2010 13:48'!
sourceSearch: aResearchString
"I'm searching in sources"
	| originalList result aString |
	aString := aResearchString.
	
	originalList := self packagesSelection.
	result := OrderedCollection new.
	
	'Scanning' 
		displayProgressAt: Display center
		from: 0 to: originalList size
		during: [:bar |  

		self useRegEx
			ifTrue:[
				originalList doWithIndex: [:class :i || temp |
					bar value: i.
					temp := class methodDict select: [:method | (method getSource asString) substrings anySatisfy: [:sub | sub matchesRegexIgnoringCase: aString]].
					temp addAll: (class classSide methodDict select: [:method | (method getSource asString) substrings anySatisfy: [:sub | sub matchesRegexIgnoringCase: aString]]).
				result addAll: temp]]
			ifFalse:[
				originalList doWithIndex: [:class :i || temp |
					bar value: i.
					temp := class methodDict select: [:method | (method getSource asString) includesSubString: aString].
					temp addAll: (class classSide methodDict select: [:method | (method getSource asString) includesSubString: aString]).
				result addAll: temp]]].
	^result.! !

!Finder methodsFor: 'private' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
testObjects: dataObjects strings: dataStrings
	| dataObjs dataStrs selectors classes didUnmodifiedAnswer |
	"Try to make substitutions in the user's inputs and search for the selector again.
1 no change to answer.
2 answer Array -> OrderedCollection.
2 answer Character -> String
4 answer Symbol or String of len 1 -> Character
	For each of these, try straight, and try converting args:
Character -> String
Symbol or String of len 1 -> Character
	Return array with result, dataObjects, dataStrings.  Don't ever do a find on the same set of data twice."

dataObjs := dataObjects.  dataStrs := dataStrings.
selectors := {#asString. #first. #asOrderedCollection}.
classes := {Character. String. Array}.
didUnmodifiedAnswer := false.
selectors withIndexDo: [:ansSel :ansInd | | do result ds answerMod | "Modify the answer object"
	answerMod := false.
	do := dataObjs copyTwoLevel.  ds := dataStrs copy.
	(dataObjs last isKindOf: (classes at: ansInd)) ifTrue: [
		((ansSel ~~ #first) or: [dataObjs last size = 1]) ifTrue: [
			do at: do size put: (do last perform: ansSel).	"asString"
			ds at: ds size put: ds last, ' ', ansSel.
			result := MethodFinder new load: do; findMessage.
			(result first beginsWith: 'no single method') ifFalse: [
				"found a selector!!"
				^ Array with: result first with: do with: ds].	
			answerMod := true]].

	selectors allButLast withIndexDo: [:argSel :argInd | | dds ddo | "Modify an argument object"
			"for args, no reason to do Array -> OrderedCollection.  Identical protocol."
		didUnmodifiedAnswer not | answerMod ifTrue: [
		ddo := do copyTwoLevel.  dds := ds copy.
		dataObjs first withIndexDo: [:arg :ind |
			(arg isKindOf: (classes at: argInd))  ifTrue: [
				((argSel ~~ #first) or: [arg size = 1]) ifTrue: [
					ddo first at: ind put: ((ddo first at: ind) perform: argSel).	"asString"
					dds at: ind put: (dds at: ind), ' ', argSel.
					result := MethodFinder new load: ddo; findMessage.
					(result first beginsWith: 'no single method') ifFalse: [
						"found a selector!!"
						^ Array with: result first with: ddo with: dds]	.	
					didUnmodifiedAnswer not & answerMod not ifTrue: [
						didUnmodifiedAnswer := true].
					]]]]].
	].
^ Array with: 'no single method does that function' with: dataObjs with: dataStrs! !

!Finder methodsFor: 'initialize-release' stamp: 'AlainPlantec 2/14/2011 21:28'!
uiClass
	"I answer the class used to create UI"
	 ^FinderUI! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/15/2010 11:21'!
useRegEx
"Getter"
"I should answer a boolean"
	 ^ useRegEx! !

!Finder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/15/2010 12:04'!
useRegEx: aBoolean
"Setter"
	useRegEx :=  aBoolean.! !

MorphTreeNodeModel subclass: #FinderNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Finder-UI'!

FinderNode subclass: #FinderClassNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Finder-UI'!

!FinderClassNode methodsFor: 'converting' stamp: 'AlainPlantec 2/13/2011 22:22'!
asString
	"Each time a classNode is drawn, if i'm looking for
	patterns, I test if my parent is a method, if it is ,I check
	if this method could be an instance the class"
	| prefix |
	prefix := ''.
	self parentNode
		ifNotNil: [:parent | 
			| string receiverString receiver |
			"My parent is a FinderMethodNode"
			string := parent item. 
			self model  finder currentSearchMode == #Examples
				ifTrue: [receiverString := string readStream
								upToAll: (self model finder findSelector: string).
					receiver := Compiler evaluate: receiverString.
					(receiver isKindOf: self item)
						ifTrue: [prefix := '*']]].
	^ self item
		ifNotNil: [:i | prefix , self item name asString]! !

!FinderClassNode methodsFor: 'private' stamp: 'CamilloBruni 2/21/2011 15:58'!
browse
	self hasParentNode ifFalse: [ ^ super browseClass ].
	self browseClass! !

!FinderClassNode methodsFor: 'private' stamp: 'CamilloBruni 2/21/2011 15:49'!
browseClass
	Browser fullOnClass: self item selector: self parentNode item! !

!FinderClassNode methodsFor: 'private' stamp: 'CamilloBruni 2/21/2011 15:56'!
browseReferences
	SystemNavigation default browseAllCallsOnClass: self item.! !

!FinderClassNode methodsFor: 'accessing' stamp: 'AlainPlantec 2/13/2011 21:03'!
childNodeClassFromItem: anItem 
	"The children (if there are some) are instances of
	FinderMethodNode (in the case of a Classes research)"
	^ FinderMethodNode ! !

!FinderClassNode methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/15/2010 12:06'!
childrenItems
	"I search the children, if I have not got any, I call my super method"
	^ (self model resultDictionary at: self item ifAbsent:[^super childrenItems])! !

!FinderClassNode methodsFor: 'event handling' stamp: 'BenjaminVanRyseghem 2/26/2011 12:46'!
keyStroke: anEvent
	(anEvent controlKeyPressed or: [ anEvent commandKeyPressed ]) ifFalse: [ ^ false ].
	anEvent keyCharacter == $h ifTrue: [ ^ self browseHierarchy ].
	anEvent keyCharacter == $N ifTrue: [ ^ self browseReferences ].
	^ super keyStroke: anEvent.! !

!FinderClassNode methodsFor: 'event handling' stamp: 'BenjaminVanRyseghem 2/26/2011 12:53'!
menu: menu shifted: aBoolean
	menu
		add: 'Hierarchy (h)' translated
		target: self
		selector: #browseHierarchy.
	menu
		add: 'References (N)' translated
		target: self
		selector: #browseReferences.
	^ super menu: menu shifted: aBoolean.! !

FinderNode subclass: #FinderMethodNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Finder-UI'!

!FinderMethodNode methodsFor: 'events handling' stamp: 'BenjaminVanRyseghem 2/26/2011 12:49'!
addInspectMenuItem: menu

	self hasParentNode ifTrue: [ ^ super addInspectMenuItem: menu ].! !

!FinderMethodNode methodsFor: 'private' stamp: 'BenjaminVanRyseghem 3/3/2011 13:29'!
browse
	self parentNode
		ifNotNil: [:parent | self browseClass].! !

!FinderMethodNode methodsFor: 'private' stamp: 'CamilloBruni 2/21/2011 15:48'!
browseClass
	Browser fullOnClass: self parentNode item selector: self item! !

!FinderMethodNode methodsFor: 'private' stamp: 'CamilloBruni 2/21/2011 15:43'!
browseImplementors
	SystemNavigation default browseAllImplementorsOf: self item.! !

!FinderMethodNode methodsFor: 'private' stamp: 'CamilloBruni 2/21/2011 15:42'!
browseSenders
	SystemNavigation default browseAllCallsOn: self item.! !

!FinderMethodNode methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/15/2010 12:13'!
childNodeClassFromItem: anItem
"I answer the class of my children nodes (if I have some)"
	^ FinderClassNode! !

!FinderMethodNode methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/15/2010 12:13'!
childrenItems
	"I search the children, if I have not got any, I call my super method"
	^ (self model resultDictionary at: self item ifAbsent:[^super childrenItems])! !

!FinderMethodNode methodsFor: 'private' stamp: 'CamilloBruni 2/21/2011 16:12'!
inspectItem
	self hasParentNode ifTrue: [ (self parentNode item methodDict at: self item) inspect ].! !

!FinderMethodNode methodsFor: 'events handling' stamp: 'BenjaminVanRyseghem 2/26/2011 12:46'!
keyStroke: anEvent
	(anEvent controlKeyPressed or: [ anEvent commandKeyPressed ]) ifFalse: [ ^ false ].
	anEvent keyCharacter == $n ifTrue: [ ^ self browseSenders ].
	anEvent keyCharacter == $m ifTrue: [ ^ self browseImplementors ].
	^ super keyStroke: anEvent.! !

!FinderMethodNode methodsFor: 'events handling' stamp: 'BenjaminVanRyseghem 2/26/2011 12:54'!
menu: menu shifted: aBoolean
	
	menu
		add: 'Implementors (m)' translated
		target: self
		selector: #browseImplementors.
		
	menu
		add: 'Senders (n)' translated
		target: self
		selector: #browseSenders.
		
	^ super menu: menu shifted: aBoolean.! !

!FinderNode methodsFor: 'events handling' stamp: 'BenjaminVanRyseghem 2/26/2011 12:53'!
addInspectMenuItem: menu

	menu
		add: 'Inspect (i)' translated
		target: self
		selector: #inspectItem.! !

!FinderNode methodsFor: 'private' stamp: 'BenjaminVanRyseghem 3/3/2011 13:30'!
browse

	self subclassResponsibility! !

!FinderNode methodsFor: 'private' stamp: 'CamilloBruni 2/21/2011 15:44'!
browseClass
	Browser fullOnClass: self item! !

!FinderNode methodsFor: 'private' stamp: 'CamilloBruni 2/21/2011 15:29'!
doubleClick
	self browse! !

!FinderNode methodsFor: 'testing' stamp: 'CamilloBruni 2/21/2011 15:47'!
hasParentNode
	^ self parentNode isNil not! !

!FinderNode methodsFor: 'private' stamp: 'CamilloBruni 2/21/2011 16:05'!
inspectItem
	self item inspect! !

!FinderNode methodsFor: 'events handling' stamp: 'BenjaminVanRyseghem 2/26/2011 12:44'!
keyStroke: anEvent
	(anEvent controlKeyPressed or: [ anEvent commandKeyPressed ]) ifFalse: [ ^ false ].
	anEvent keyCharacter == $b ifTrue: [ ^ self browse ].
	anEvent keyCharacter == $i ifTrue: [ ^ self inspectItem ].! !

!FinderNode methodsFor: 'events handling' stamp: 'BenjaminVanRyseghem 2/26/2011 12:51'!
menu: menu shifted: aBoolean
	menu
		add: 'Browse (b)' translated
		target: self
		selector: #browse.
	self addInspectMenuItem: menu.
		
	menu addLine.
	menu
		add: 'Expand All' translated
		target: self model
		selector: #expandAll.
	menu
		add: 'Collapse All' translated
		target: self model
		selector: #collapseAll.
	^menu! !

MorphTreeModel subclass: #FinderUI
	instanceVariableNames: 'finder searchingTextArea sourceTextArea useRegExCheckbox forceSearch'
	classVariableNames: 'SearchedTextList'
	poolDictionaries: ''
	category: 'Finder-UI'!

FinderUI class
	instanceVariableNames: 'searchedTextListMaxSize'!

FinderUI class
	instanceVariableNames: 'searchedTextListMaxSize'!

!FinderUI class methodsFor: 'event subscriptions' stamp: 'BenjaminVanRyseghem 2/25/2011 17:05'!
doAllSubscriptionsOn: aFinder to: anInstance
	self subscribesUpdateSourceCodeOn: aFinder to: anInstance.
	self subscribesResetEnvironmentOn: aFinder to: anInstance.
	self subscribesEnableUseRegExOn: aFinder to: anInstance.
	self subscribesDisableUseRegExOn: aFinder to: anInstance.
	self subscribesUpdateSelectedMethodOn: aFinder to: anInstance.
	self subscribesUpdateSelectedClassOn: aFinder to: anInstance.
	self subscribesUpdateListOn: aFinder to: anInstance! !

!FinderUI class methodsFor: 'settings' stamp: 'BenjaminVanRyseghem 12/16/2010 15:41'!
finderUISettingsOn: aBuilder
	<systemsettings>
		(aBuilder group: #finderUI)
		target: self;
		label: 'Finder' translated;
		description: 'Settings related to the finder' translated;
			with: [
				(aBuilder setting: #searchedTextListMaxSize)
				label: 'Size of the History' translated]! !

!FinderUI class methodsFor: 'initialize-release' stamp: 'AlainPlantec 2/13/2011 22:20'!
initialize
	searchedTextListMaxSize := 15! !

!FinderUI class methodsFor: 'explanations' stamp: 'bvr 9/19/2010 18:41'!
methodFinderExplanation
	"The comment in the bottom pane"

	false ifTrue: [MethodFinder methodFor: #( (4 3) 7  (0 5) 5  (5 5) 10)].
		"to keep the method methodFor: from being removed from the system"

	^ 'Type a fragment of a selector in the top pane.  Accept it.

Or, use an example to find a method in the system.  Type receiver, args, and answer in the top pane with periods between the items.  3. 4. 7

Or, in this pane, use examples to find a method in the system.  Select the line of code and choose "print it".  

	MethodFinder methodFor: #( (4 3) 7  (0 5) 5  (5 5) 10).
This will discover (data1 + data2).

You supply inputs and answers and the system will find the method.  Each inner array is a list of inputs.  It contains the receiver and zero or more arguments.  For Booleans and any computed arguments, use brace notation.

	MethodFinder methodFor: { {1. 3}. true.  {20. 10}. false}.
This will discover the expressions (data1 < data2), (data2 > data1), and many others.

	MethodFinder methodFor: { {''29 Apr 1999'' asDate}. ''Thursday''.  
		{''30 Apr 1999'' asDate}. ''Friday'' }.
This will discover the expression (data1 weekday)

Receiver and arguments do not have to be in the right order.
See MethodFinder.verify for more examples.'! !

!FinderUI class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 2/25/2011 17:04'!
on: aFinder
	| instance |
	instance := self new finder: aFinder.
	self doAllSubscriptionsOn: aFinder to: instance.
	^instance.! !

!FinderUI class methodsFor: 'accessing' stamp: 'AlainPlantec 2/14/2011 01:00'!
searchedTextListMaxSize

	^ searchedTextListMaxSize ifNil: [searchedTextListMaxSize := 15]! !

!FinderUI class methodsFor: 'accessing' stamp: 'bvr 9/19/2010 19:19'!
searchedTextListMaxSize: anInteger

	self allInstancesDo: [:each | each searchedTextListMaxSize: anInteger].
	searchedTextListMaxSize := anInteger! !

!FinderUI class methodsFor: 'event subscriptions' stamp: 'BenjaminVanRyseghem 2/25/2011 17:04'!
subscribesDisableUseRegExOn: aFinder to: anInstance.
	aFinder
		when: #disableUseRegEx
		send: #disableUseRegEx
		to: anInstance.! !

!FinderUI class methodsFor: 'event subscriptions' stamp: 'BenjaminVanRyseghem 2/25/2011 17:04'!
subscribesEnableUseRegExOn: aFinder to: anInstance
	aFinder
		when: #enableUseRegEx
		send: #enableUseRegEx
		to: anInstance.! !

!FinderUI class methodsFor: 'event subscriptions' stamp: 'AlainPlantec 2/13/2011 22:20'!
subscribesResetEnvironmentOn: aFinder to: anInstance 
	aFinder
		when: #resetEnvironment
		send: #resetEnvironment
		to: anInstance! !

!FinderUI class methodsFor: 'event subscriptions' stamp: 'BenjaminVanRyseghem 2/25/2011 17:05'!
subscribesUpdateListOn: aFinder to: anInstance
	aFinder
		when: #updateResultDictionary
		send: #updateList
		to: anInstance.! !

!FinderUI class methodsFor: 'event subscriptions' stamp: 'BenjaminVanRyseghem 2/25/2011 17:04'!
subscribesUpdateSelectedClassOn: aFinder to: anInstance
	aFinder
		when: #updateSelectedClass
		send: #updateSelectedClass
		to: anInstance.! !

!FinderUI class methodsFor: 'event subscriptions' stamp: 'BenjaminVanRyseghem 2/25/2011 17:04'!
subscribesUpdateSelectedMethodOn: aFinder to: anInstance
	aFinder
		when: #updateSelectedMethod
		send: #updateSelectedMethod
		to: anInstance.! !

!FinderUI class methodsFor: 'event subscriptions' stamp: 'BenjaminVanRyseghem 2/25/2011 17:04'!
subscribesUpdateSourceCodeOn: aFinder to: anInstance
	aFinder
		when: #updateSourceCode
		send: #updateSourceCode
		to: anInstance.! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 12/20/2010 10:11'!
addAllItems: aWindow
	| container toolbar height  btnFont  btnToolbar cst |
	toolbar := self buildSearchModeToolBar: aWindow.
	height := toolbar minExtent y.
	cst := 15.
	
	aWindow 
		addMorph: toolbar
		fullFrame: (LayoutFrame 
						fractions: (0 @ 0 corner: 1 @ 0)
						offsets: (0 @ 0 corner: 0 @ height)).
	aWindow
		addMorph: self buildPackagesTree buildContents
		fullFrame: (LayoutFrame 
			fractions: (0@0 corner: 1@0.58)
			offsets: (0@height corner: 0@0)).
	btnFont := StandardFonts buttonFont.
	btnToolbar := self buildBrowseToolBar: aWindow.
	aWindow
		addMorph: btnToolbar
		fullFrame: (LayoutFrame
						fractions: (0@0.58 corner: 1@0.58)
						offsets:(0@0 corner: 0@(btnFont height +cst))).
	aWindow 
		addMorph: self sourceTextArea
		fullFrame: (LayoutFrame
						fractions: (0@0.58 corner: 1@1)
						offsets:(0@(btnFont height +cst) corner: 0@0)).
! !

!FinderUI methodsFor: 'accessing' stamp: 'AlainPlantec 2/13/2011 21:28'!
addToSearchedTextList: aString 
	self searchedTextList: (self searchedTextList
			remove: aString
			ifAbsent: [];
			 yourself).
	self searchedTextList size = self searchedTextListMaxSize
		ifTrue: [self searchedTextList removeLast].
	self searchedTextList addFirst: aString ! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 9/17/2010 01:57'!
allClassesButtonAction

	self finder packagesSelection: self environment.! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'AlainPlantec 2/13/2011 22:31'!
allClassesButtonLabel
	^ 'All Packages'! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
allClassesButtonState
	^false! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 11/11/2010 23:49'!
autoSelection: aText
	| aString firstIndex interval string size |
	(self searchingString isEmpty | self useRegEx )ifTrue: [^self].
	aString := aText asString asLowercase.
	string := self searchingString asLowercase.
	(string includes: $*)
		ifTrue: [| list lastIndex|
			list := string subStrings: '*'.
			firstIndex := aString findString: list first.
			lastIndex := (aString findString: list last startingAt: firstIndex) + list last size - 1.
			interval := firstIndex to: lastIndex]
		ifFalse: [| index |
			firstIndex := aString findString: string.
			size := string size.
			interval := firstIndex to: (firstIndex + size - 1)].
		
	firstIndex = 0 ifFalse: [self sourceTextArea setSelection: interval]! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 3/3/2011 13:28'!
browseButtonAction

	self selectedNode browse.! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
browseButtonLabel
	^'Browse'! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 3/3/2011 13:29'!
browseButtonState
	^self selectedClass isNil or: [self selectedMethod isNil]! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
buildAllClassesButton
	^ PluggableButtonMorph 
			on: self 
			getState: #allClassesButtonState 
			action: #allClassesButtonAction 
			label: #allClassesButtonLabel! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
buildAllTextArea
	self buildSearchingTextArea.
	self buildSourceTextArea.! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 9/15/2010 02:03'!
buildBrowseButton
	^ (PluggableButtonMorph 
			on: self 
			getState: #browseButtonState 
			action: #browseButtonAction 
			label: #browseButtonLabel)
			hResizing: #spaceFill! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 9/16/2010 22:59'!
buildBrowseToolBar: aWindow
	| toolbar |
	toolbar := aWindow newToolbar: {
		self buildBrowseButton.
		self buildSendersButton.
		self buildImplementorsButton.
		self buildVersionsButton.
		self buildInheritanceButton.
		self buildHierarchyButton}.
	^toolbar hResizing: #shrinkWrap! !

!FinderUI methodsFor: 'text areas behavior' stamp: 'BenjaminVanRyseghem 3/4/2011 12:18'!
buildDescriptionOf: aClass

	| stream |
	stream := WriteStream on: ''.
	stream 
		nextPutAll: aClass definition.
	aClass hasComment
		ifTrue: [
			stream
				nextPut: Character cr;
				nextPut: Character cr;
				nextPut: Character cr;
				nextPutAll: aClass comment].

	^ stream contents! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
buildEnvironmentButton
	^ (PluggableButtonMorph 
			on: self 
			getState: #environmentButtonState 
			action: #environmentButtonAction 
			label: #environmentButtonLabel)
			beSticky! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 9/15/2010 02:03'!
buildHierarchyButton
	^ (PluggableButtonMorph 
			on: self 
			getState: #hierarchyButtonState 
			action: #hierarchyButtonAction 
			label: #hierarchyButtonLabel)
			hResizing: #spaceFill! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 9/15/2010 02:04'!
buildImplementorsButton
	^ (PluggableButtonMorph 
			on: self 
			getState: #implementorsButtonState 
			action: #implementorsButtonAction 
			label: #implementorsButtonLabel)
			hResizing: #spaceFill! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 9/15/2010 02:04'!
buildInheritanceButton
	^ (PluggableButtonMorph 
			on: self 
			getState: #inheritanceButtonState 
			action: #inheritanceButtonAction 
			label: #inheritanceButtonLabel)
			hResizing: #spaceFill! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 2/25/2011 17:03'!
buildPackagesTree
	| tree | 
	tree := (MorphTreeMorph on: self) doubleClickSelector: #doubleClick;
				beSingle;
				keystrokeActionSelector: #keyStroke:;
				autoDeselection: false;
				rowInset: 4;
				columnInset: 4;
				getMenuSelector: #menu:shifted:;
				rowColorForEven: Color lightGray muchLighter odd: Color white.

	^ tree! !

!FinderUI methodsFor: 'items creation' stamp: 'AlainPlantec 2/13/2011 22:30'!
buildRegExChooser
	| string btnHeight btnWidth |
	string := 'Regexp'.
	btnHeight := StandardFonts buttonFont height + 8.
	btnWidth := 0.
	string do: [:c | btnWidth := btnWidth + (StandardFonts buttonFont widthOf: c)].
	^useRegExCheckbox := (CheckboxMorph 
									on: self finder
									selected: #useRegEx
									changeSelected: #useRegEx:)
									beCheckbox;
									vResizing: #rigid;
									height: btnHeight;
									width: btnWidth + 30;
									label: string! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 2/17/2011 11:06'!
buildSearchButton
	^ (PluggableButtonMorph 
			on: self 
			getState: #searchButtonState 
			action: #searchButtonAction 
			label: #searchButtonLabel)
			hResizing: #shrinkWrap! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 2/17/2011 11:06'!
buildSearchModeDropListIn: aWindow
			^(UITheme current 
				newDropListIn: aWindow
				for:self				
				list: #searchModesList
				getSelected: #currentSearchMode
				setSelected: #currentSearchMode:
				getEnabled: nil
				useIndex: false 
				help: self searchModeHelpText) hResizing: #rigid; width: 120; yourself
! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 2/17/2011 11:05'!
buildSearchModeToolBar: aWindow
	^aWindow newToolbar: {
		self buildSearchingTextArea.
		self buildRegExChooser.
		self buildSearchModeDropListIn: aWindow.
		self buildEnvironmentButton.
		self buildAllClassesButton.
		self buildSearchButton}.! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 9/13/2010 17:28'!
buildSearchingTextArea
	^ searchingTextArea := 
		(EditableDropListMorph
				on: self
				list: #searchedTextList
				selected: #searchingString
				changeSelected: #searchingAccept:
				useIndex: false 
				addToList: #addToSearchedTextList:
				class: String
				getEnabled: nil) hResizing: #spaceFill; 
				ghostText: 'Hit return to accept' translated.

	! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 9/15/2010 02:04'!
buildSendersButton
	^ (PluggableButtonMorph 
			on: self 
			getState: #sendersButtonState 
			action: #sendersButtonAction 
			label: #sendersButtonLabel)
			hResizing: #spaceFill! !

!FinderUI methodsFor: 'items creation' stamp: 'AlainPlantec 2/13/2011 21:52'!
buildSourceTextArea
	sourceTextArea := PluggableTextMorph
				on: self
				text: #sourceCode
				accept: #compileSource: 
				readSelection: nil
				menu: #msgPaneMenu:shifted:.
	self updateSourceCode.
	^ sourceTextArea! !

!FinderUI methodsFor: 'items creation' stamp: 'BenjaminVanRyseghem 9/15/2010 02:04'!
buildVersionsButton
	^ (PluggableButtonMorph 
			on: self 
			getState: #versionsButtonState 
			action: #versionsButtonAction 
			label: #versionsButtonLabel)
			hResizing: #spaceFill! !

!FinderUI methodsFor: 'text areas behavior' stamp: 'BenjaminVanRyseghem 9/17/2010 02:50'!
collectFromPackages: aCollection
	self packagesSelection: (self environment select: [:each | aCollection includes: each category ])! !

!FinderUI methodsFor: 'text areas behavior' stamp: 'bvr 9/19/2010 18:55'!
compileSource: aString
	| association class method |
	self selectedClass isNil | self selectedMethod isNil
		ifTrue:[
			self sourceTextArea hasUnacceptedEdits: false.
			self changed: #sourceCode.
			^self].
	class := self selectedClass.
	method := self selectedMethod.
	class compile: (aString) classified: ((class >> method) category).
	self changed: #sourceCode.! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/18/2010 14:41'!
constructPackagesSet
	| list result |
	list := (self environment difference: self packagesSelection).
	result := OrderedCollection new.
	'Building Packages'
	 	displayProgressAt: Display center
		from: 0 to: list size
		during:[ :bar | 
			list doWithIndex: [:each :i | 
				bar value: i.
				result add: each category]].
			
	^result asSet asSortedCollection: [:a :b | a<b]! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/17/2010 03:02'!
constructPackagesSet: aList
	| result |
	result := OrderedCollection new.
	'Building Packages'
	 	displayProgressAt: Display center
		from: 0 to: aList size
		during:[ :bar | 
			aList doWithIndex: [:each :i | 
				bar value: i.
				result add: each category]].
	
		^(result reject: [:each | each isNil]) asSet asSortedCollection: [:a :b | a<b].! !

!FinderUI methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/14/2010 17:44'!
currentSearchMode
	^self finder currentSearchMode! !

!FinderUI methodsFor: 'accessing' stamp: 'bvr 9/19/2010 18:27'!
currentSearchMode: aSymbol
	
	self finder searchingStringSilently: self searchingTextArea contentMorph textMorph text asString.
	self searchingTextArea contentMorph textMorph hasUnacceptedEdits:false.
	self finder currentSearchMode: aSymbol.
	.! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 2/17/2011 10:54'!
defaultExplanation
^
'The Finder can be used in 4 different ways:
	- Selectors: your research is done among selectors
	- Classes : your research is done among classes names
	- Source : your research is done among all the source code
	- Examples : your research uses the Method Finder behavior 
			   (for further informations, print ''FinderUI methodFinderExplanation'')
			
In these four modes, you can also tick the ''Use RegEx'' checkbox.
If you pick this box, your search will be done using regular expressions instead of just matching.

The ''Select classes'' button opened a dialog window  to select which classes will be used for the search.
The ''All classes'' button is used to reset the classes selection.'! !

!FinderUI methodsFor: 'display' stamp: 'BenjaminVanRyseghem 9/15/2010 02:47'!
defaultWindowLabel
	^'Finder'.! !

!FinderUI methodsFor: 'checkbox' stamp: 'BenjaminVanRyseghem 9/15/2010 01:47'!
disableUseRegEx
	useRegExCheckbox isSelected
		ifTrue: [ useRegExCheckbox toggleSelected].
	useRegExCheckbox enabled: false.
! !

!FinderUI methodsFor: 'events handling' stamp: 'BenjaminVanRyseghem 2/25/2011 16:48'!
doubleClick
	self selectedNode doubleClick! !

!FinderUI methodsFor: 'checkbox' stamp: 'BenjaminVanRyseghem 9/15/2010 01:41'!
enableUseRegEx
	useRegExCheckbox 
		enabled: true;
		updateEnabled! !

!FinderUI methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/13/2010 16:12'!
environment

	^ self finder environment.! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 9/16/2010 01:16'!
environmentButtonAction
	self openPackageChooser! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'AlainPlantec 2/13/2011 22:48'!
environmentButtonLabel
	^ 'Choose Packages'! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
environmentButtonState
	^false.! !

!FinderUI methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/13/2010 16:02'!
finder
	^finder! !

!FinderUI methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/13/2010 16:03'!
finder: aFinder
	finder := aFinder! !

!FinderUI methodsFor: 'searching' stamp: 'BenjaminVanRyseghem 11/11/2010 23:54'!
forSelectorsDo: selectorBlock forClassNamesDo: classNamesBlock forSourceDo: sourceBlock forPatternDo: patternBlock

	self currentSearchMode = #Selectors
		ifTrue: [^selectorBlock value].
	self currentSearchMode = #Classes
		ifTrue: [^classNamesBlock value].
	self currentSearchMode = #Source
		ifTrue: [^sourceBlock value].
	self currentSearchMode = #Examples
		ifTrue: [^patternBlock value].! !

!FinderUI methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 3/2/2011 16:41'!
forceSearch

	 ^ forceSearch ifNil: [forceSearch := false].! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/15/2010 11:24'!
hierarchy
	
		self selectedClass ifNotNilDo: [ :class |
			self selectedMethod
				ifNil: [SystemNavigation new browseHierarchy: class]
				ifNotNilDo: [:selector | 
					SystemNavigation new browseHierarchy:class selector: selector]]! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
hierarchyButtonAction
	self hierarchy.! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
hierarchyButtonLabel
	^'Hierarchy'! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
hierarchyButtonState
	^self selectedClass isNil.! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/15/2010 11:24'!
implementors

			self selectedMethod ifNotNilDo: [ :selector |
		 		SystemNavigation new browseImplementorsOf: selector name: 'Implementors of ', selector asString autoSelect: nil]! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
implementorsButtonAction
	self implementors.! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
implementorsButtonLabel
	^'Implementors'! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
implementorsButtonState
	^self selectedMethod isNil | self isClassNamesSymbol.! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/15/2010 11:24'!
inheritance
	
		self selectedClass ifNotNilDo: [ :class |.
				self selectedMethod ifNotNilDo: [:selector | 
					SystemNavigation new methodHierarchyBrowserForClass: class selector: selector]]! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
inheritanceButtonAction
	self inheritance.! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
inheritanceButtonLabel
	^'Inheritance'! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
inheritanceButtonState
	^self selectedClass isNil | self isClassNamesSymbol.! !

!FinderUI methodsFor: 'display' stamp: 'AlainPlantec 2/13/2011 21:17'!
initialExtent
	^700 @ 500! !

!FinderUI methodsFor: 'mode list' stamp: 'BenjaminVanRyseghem 9/14/2010 17:46'!
isClassNamesSymbol
	^self currentSearchMode = #Classes! !

!FinderUI methodsFor: 'mode list' stamp: 'BenjaminVanRyseghem 9/18/2010 15:30'!
isPatternSymbol
	^self finder isPatternSymbol! !

!FinderUI methodsFor: 'mode list' stamp: 'BenjaminVanRyseghem 9/15/2010 00:10'!
isSelectorsSymbol
	^self finder isSelectorsSymbol! !

!FinderUI methodsFor: 'mode list' stamp: 'BenjaminVanRyseghem 9/14/2010 23:54'!
isSourceSymbol
	^self finder isSourceSymbol! !

!FinderUI methodsFor: 'events handling' stamp: 'StephaneDucasse 3/12/2011 15:07'!
keyStroke: event
	self selectedNode ifNotNil: [:node | node keyStroke: event]! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
labelFont
	^StandardFonts defaultFont! !

!FinderUI methodsFor: 't - accessing' stamp: 'BenjaminVanRyseghem 2/26/2011 12:53'!
menu: menu shifted: b 
	^ self selectedNode menu: menu shifted: b ! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
msgPaneMenu: aMenu shifted: shifted
	^ aMenu labels: 
'find...(f)
find again (g)
set search string (h)
do again (j)
undo (z)
copy (c)
cut (x)
paste (v)
paste...
do it (d)
print it (p)
inspect it (i)
explore it (I)
debug it
accept (s)
cancel (l)
implementors (m)
methods containing (E)
senders (n)' 
		lines: #(0 3 5 9 14)
		selections: #(find findAgain setSearchString again undo copySelection cut paste pasteRecent doIt printIt inspectIt exploreIt debugIt accept cancel implementorsOfIt methodSourceContainingIt sendersOfIt)! !

!FinderUI methodsFor: 'display' stamp: 'AlainPlantec 2/13/2011 22:03'!
open
	| window |
	window := StandardWindow new model: self.
	self addAllItems: window.
	window setLabel: self defaultWindowLabel.
	window openInWorld.
	self searchingTextArea contentMorph takeKeyboardFocus! !

!FinderUI methodsFor: 'private' stamp: 'AlainPlantec 2/13/2011 21:07'!
openPackageChooser
	(DialogItemsChooser
		on: self
		unselectedItems: self constructPackagesSet
		selectedItems: (self constructPackagesSet: self packagesSelection)
		selectedItemsSetterSelector: #collectFromPackages: 
		title: 'Package Chooser'
		unselectedItemsLabel: 'Unselected Packages' translated
		selectedItemsLabel: 'Selected Packages' translated) openInWorld! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/16/2010 21:09'!
packagesSelection
	^self finder packagesSelection! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/17/2010 02:16'!
packagesSelection: aCollection

	self finder packagesSelection: aCollection.! !

!FinderUI methodsFor: 'text areas behavior' stamp: 'AlainPlantec 2/13/2011 22:28'!
removeStyler
	sourceTextArea
		ifNotNil: [sourceTextArea
				styler: (NullTextStyler new view: sourceTextArea)] ! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
resetEnvironment
	self triggerEvent: #resetEnvironment! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/15/2010 01:21'!
resetSearchedTextList
	self searchedTextList removeAll.! !

!FinderUI methodsFor: 'tree behavior' stamp: 'BenjaminVanRyseghem 9/14/2010 16:35'!
resultDictionary 
	^self finder resultDictionary ! !

!FinderUI methodsFor: 'tree behavior' stamp: 'BenjaminVanRyseghem 9/18/2010 15:08'!
rootItems
	^self resultDictionary keys sort:[:a :b | a name  < b name]! !

!FinderUI methodsFor: 't - accessing' stamp: 'AlainPlantec 2/13/2011 21:23'!
rootNodeClassFromItem: anItem 
	"To have the good class for my nodes, I ask my owner,
	because he is the only one who knows his state"
	^ self 
		forSelectorsDo: [FinderMethodNode]
		forClassNamesDo: [FinderClassNode]
		forSourceDo: [FinderMethodNode]
		forPatternDo: [FinderMethodNode] ! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 2/17/2011 11:20'!
searchButtonAction
	
	forceSearch := true.
	self searchingTextArea contentMorph acceptTextInModel.! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
searchButtonLabel
	^'Search'! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
searchButtonState
	^false.! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/13/2010 16:46'!
searchModeHelpText
	^'Choose the mode for the search'! !

!FinderUI methodsFor: 'mode list' stamp: 'BenjaminVanRyseghem 11/11/2010 23:32'!
searchModesList
	^#(Selectors Classes Source Examples)! !

!FinderUI methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/14/2010 14:25'!
searchedTextList
	^SearchedTextList ifNil: [ SearchedTextList := OrderedCollection new]! !

!FinderUI methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/16/2010 14:55'!
searchedTextList: aCollection

	SearchedTextList := aCollection! !

!FinderUI methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/18/2010 13:42'!
searchedTextListMaxSize
	^self class searchedTextListMaxSize! !

!FinderUI methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/18/2010 16:39'!
searchedTextListMaxSize: anInteger
	[self searchedTextList size > anInteger]
		whileTrue: [ self searchedTextList removeLast].
	self changed: #searchedTextList.! !

!FinderUI methodsFor: 'text areas behavior' stamp: 'BenjaminVanRyseghem 2/17/2011 11:21'!
searchingAccept: aText

	self forceSearch
		ifFalse: [aText asString = self searchingString ifTrue: [^self]].
	self searchingString: aText asString.
	forceSearch := false.! !

!FinderUI methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/13/2010 16:06'!
searchingString
	^self finder searchingString! !

!FinderUI methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 2/23/2011 15:27'!
searchingString: aString

	^self finder searchingString: aString! !

!FinderUI methodsFor: 'accessing' stamp: 'AlainPlantec 2/13/2011 21:48'!
searchingTextArea
	^searchingTextArea ifNil: [self buildSearchingTextArea]! !

!FinderUI methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/13/2010 16:11'!
selectedClass
	^ self finder selectedClass! !

!FinderUI methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/13/2010 16:13'!
selectedMethod
	^ self finder selectedMethod! !

!FinderUI methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 2/25/2011 16:42'!
selection: anObject
	
	super selection: anObject.
	self finder selection: anObject! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 2/23/2011 15:27'!
senders

	self selectedMethod 
		ifNotNilDo: [ :selector |.
			 	SystemNavigation new browseSendersOf: selector name: 'Senders of ', selector asString autoSelect: selector asString].! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
sendersButtonAction
	self senders.! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
sendersButtonLabel
	^'Senders'! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
sendersButtonState
	^self selectedMethod isNil | self isClassNamesSymbol.! !

!FinderUI methodsFor: 'text areas behavior' stamp: 'BenjaminVanRyseghem 3/4/2011 15:27'!
sourceCode

	^ self selectedClass isNil
		ifTrue: [
			self removeStyler.
			self defaultExplanation]
		ifFalse:[
			self selectedMethod isNil
				ifTrue: [
					self removeStyler.
					self buildDescriptionOf: self selectedClass]
				ifFalse:[
					| method |
					self useDefaultStyler.
					method := self isPatternSymbol
								ifTrue: [self finder findSelector: self selectedMethod]
								ifFalse: [self selectedMethod].
					self sourceTextArea styler classOrMetaClass: self selectedClass.
					(self selectedClass >> method) getSource]]! !

!FinderUI methodsFor: 'accessing' stamp: 'AlainPlantec 2/13/2011 21:41'!
sourceTextArea
	^sourceTextArea ifNil: [self buildSourceTextArea]! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 2/25/2011 17:07'!
updateList
	"I update the list of my tree"
	
	self changed: #rootNodes.! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/15/2010 02:16'!
updateSelectedClass
	self changed: #selectedClass! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/15/2010 02:16'!
updateSelectedMethod
	self changed: #selectedMethod! !

!FinderUI methodsFor: 'text areas behavior' stamp: 'bvr 9/19/2010 18:51'!
updateSourceCode
	| code |
	code := self sourceCode.
	self sourceTextArea setText: code.
	self sourceTextArea setSelection:(0 to: 0).
	((self isSourceSymbol) & (code ~~ self defaultExplanation))
		ifTrue: [	self autoSelection: code]! !

!FinderUI methodsFor: 'text areas behavior' stamp: 'BenjaminVanRyseghem 3/4/2011 15:22'!
useDefaultStyler
	self sourceTextArea useDefaultStyler! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/15/2010 02:45'!
useRegEx
	^ self finder useRegEx! !

!FinderUI methodsFor: 'items creation' stamp: 'AlainPlantec 2/13/2011 21:44'!
useRegExCheckbox
	^ useRegExCheckbox ifNil: [self buildRegExChooser]
! !

!FinderUI methodsFor: 'private' stamp: 'BenjaminVanRyseghem 9/15/2010 11:25'!
versions
	self selectedClass ifNotNilDo: [ :class |
		self selectedMethod ifNotNilDo: [:selector |
			VersionsBrowser browseVersionsForClass: class selector: selector]]! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
versionsButtonAction
	self versions.! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
versionsButtonLabel
	^'Versions'! !

!FinderUI methodsFor: 'buttons behavior' stamp: 'BenjaminVanRyseghem 8/30/2010 3:44'!
versionsButtonState
	^self selectedClass isNil | self isClassNamesSymbol.! !
FinderUI initialize!
